@import url(http://fonts.googleapis.com/css?family=Open+Sans:400italic,400,300,700);
@import url(http://fonts.googleapis.com/css?family=Ubuntu+Mono);

@actionColor: #a00;
@actionColorBlack: #ddd;

body,
input,
textarea,
select {
    font: 400 1em Open Sans,sans-serif;
}

body {
    background: #222;
    color: #999;
    font-size: .9em;
    margin: 0;
    padding: 0;
}

h1,
h2,
h3 {
    font: 300 2.5em Open Sans,sans-serif;
    margin: 0;
    padding: 25px 0 15px;
    text-transform: uppercase;
}

h2 {
    font-size: 1.75em;
}

h3 {
    font-size: 1.25em;
    padding: 20px 0 10px;
}

pre,
code {
    font: 400 1em Ubuntu Mono,monospace;
}

a {
    color: @actionColorBlack;
    cursor: pointer;
    text-decoration: none;

    &:hover {
        text-decoration: underline;
    }
}

b {
    font-weight: 700;
}

footer {
    font-size: .8em;
    margin: 0 auto;
    padding: 40px;
    text-align: center;
    width: 600px;

    img {
        margin: 10px 20px;
    }
}

#content {
    background: #fff;
    color: #222;
    padding: 40px;

    a {
        color: @actionColor;
    }
}

#abstract {
    margin: 10px;
}

#query-form {
    font-size: 1.5em;
    margin: 5px 20px;
    position: absolute;

    input {
        line-height: 30px;
        margin: 0;
        padding: 2px 5px;

        &[type=submit] {
            margin-right: 40px;
            padding: 3px 10px;
        }
    }

    label {
        background: #fff;
        margin-left: 20px;
    }

    .options {
        font-size: .75em;
    }
}

#results {
    border: dotted 1px #aaa;
    font: 400 1em Ubuntu Mono,monospace;
    margin: 40px 0 20px;
    padding: 30px 20px 20px;
    white-space: pre;

    .intro {
        padding: 50px;
        text-align: center;
    }

    noscript {
        background: #ffb;
        display: block;
        font-weight: 700;
        padding: 5px 10px;
    }

    .json-key {
        color: #888;
    }

    .json-string {
        color: @actionColor;
    }

    .json-value {
        color: #a08;
    }
}

#illustrations {
    a {
        display: block;
        float: left;
        padding: 10px 5px;
        width: 350px;

        &:hover {
            background: #fee;
            text-decoration: none;
        }
    }

    .info {
        color: #999;
        display: block;
        font-size: .8em;
    }

    .credentials {
        background: #fcc;
        border-radius: 4px;
        color: #444;
        margin-left: 2px;
        padding: 0 2px;

        span {
            padding: 0 2px;
        }

        &.empty {
            background: #ddd;
            color: #777;
        }
    }
}

#documentation {
    clear: both;
    padding-top: 20px;
}
