class NodeNotFoundException(Exception):
    pass


class AuthenticationException(Exception):
    pass


class FormatterException(Exception):
    pass


class InvalidQueryException(Exception):
    pass


class CircularInheritanceException(Exception):
    pass
